/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   for operating on email addresses
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "address.h"
#include "xmalloc.h"
#include "hash.h"
#include "elmo.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/

static char  get_sex (const char *name, char **last);
static char *get_name (char *addr, char **email);

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

/**
 * This function may get something like:
 *   - Ania Kowalska
 *   - Grzesiu
 *   - manka_23@onet.pl
 */
static char
get_sex (const char *name, char **last)
{
  char *seek;

  if (!name)
    return 'M';

  *last = NULL;
  seek  = strchr (name, ' ');

  /* option 1 */
  if (seek && seek > name){
    *last = seek + 1;
    while (**last && !isalnum (**last))
      (*last)++;
    if (**last == '\0')
      *last = seek;
    if (*(seek - 1) == 'a' || *(seek - 1) == 'A')
      return 'F';
    else
      return 'M';
  }
  /* option 3 */
  else if ((seek = strchr (name, '@')) != NULL) {
    do {
      seek--;
    } while (seek > name && !isalpha (*seek));
    if (*seek == 'a' || *seek == 'A')
      return 'F';
    else
      return 'M';
  }
  /* option 2  --  fallback */
  else {
    size_t len = strlen (name);

    if (name[len - 1] == 'a' || name[len - 1] == 'A')
      return 'F';
    else
      return 'M';
  }
}


static char *
get_name (char *addr, char **email)
{
  char *seek;
  char *end;
  char *result;

  if (!addr){
    *email = NULL;
    return NULL;
  }

  /*
   * option 1:  John Doe <john@doe.com>
   */
  seek = strchr (addr, '<');
  if (seek == addr){
    *email = addr + 1;
    return xstrdup (addr);
  }

  if (seek){

    *email = seek + 1;
    do
      seek--;
    while (seek > addr && isspace (*(seek - 1)));

    result = xmalloc (seek - addr + 1);
    result[seek - addr] = '\0';
    memcpy (result, addr, seek - addr);
    return result;
  }

  /*
   * option 2:  john@doe.com (John Doe)
   */
  *email = addr;
  while (isspace (**email))
    (*email)++;
  seek   = strchr (addr, '(');
  if (!seek)
    return xstrdup (addr);
  seek++;
  end = strchr (addr, ')');
  if (!end)
    return xstrdup (addr);

  result = xmalloc (end - seek + 1);
  result[end - seek] = '\0';
  memcpy (result, seek, end - seek);
  return result;
}



static char *
get_email (char *s)
{
  char  old;
  char *seek = s;
  char *result;

  if (s == NULL)
    return NULL;

  while (isspace (*s))
    s++;
  
  while (*seek){
    if (isspace (*seek) || *seek == '>')
      break;
    seek++;
  }
  old    = *seek;
  *seek  = '\0';
  result = xstrdup (s);
  *seek  = old;

  return result;
}



static int
have_special (const char *str)
{
  while (str && *str){

    if (*str == '.'    || *str == '@' || *str == ')'  || *str == '('
        || *str == '<' || *str == '>' || *str == '['  || *str == ']'
        || *str == ':' || *str == ';' || *str == '\\' || *str == ',')
      return 1;
    str++;
  }
  return 0;
}



static void
make_full (address_t *addr)
{
  size_t  elen   = strlen (addr->email);
  size_t  nlen   = strlen (addr->name);
  size_t  dquote = 0;
  char   *result;

  if (have_special (addr->name))
    dquote = 2;

  /*                         name   _   <   email  >  \0 */
  result = xmalloc (dquote + nlen + 1 + 1 + elen + 1 + 1);

  memcpy (result + dquote / 2, addr->name, nlen);
  memcpy (result + dquote + nlen + 2, addr->email, elen);

  if (dquote){
    result[0]        = '"';
    result[nlen + 1] = '"';
  }
  
  result[nlen + dquote] = ' ';
  result[nlen + dquote + 1] = '<';
  result[nlen + dquote + 1 + 1 + elen] = '>';
  result[nlen + dquote + 1 + 1 + elen + 1] = '\0';

  addr->full = result;
}



static char *
get_last (char *name)
{
  char *seek = strchr (name, ' ');

  if (seek == NULL)
    return NULL;

  do {
    seek++;
  }
  while (*seek && isspace (*seek));
  if (*seek == '\0')
    seek = name;
  return seek;
}



/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


/**
 * Full name (i.e. "John Doe <john@doe.com>") is a key in hash table.
 * Once I wanted email address to be a key - field but it failed.
 * Remember few things:
 *   - Do NOT alter address fields.  This will influence address book.
 *   - Do NOT delete any address.  They are all deleted at exit.
 */
address_t *
address_insert (address_t *addr)
{
  entry_t *entry;

  if (addr == NULL || addr->full == NULL)
    return NULL;
  
  entry = htable_insert (address_table, addr->full, NULL);
  if (!entry->content){
    entry->content = addr;
  }
  else {
    address_destroy (addr);
  }
  
  return entry->content;
}



address_t *
address_create (char *addr)
{
  char      *email;
  address_t *result = xcalloc (1, sizeof (address_t));

  while (isspace (*addr))
    addr++;
    
  result->name  = get_name (addr, &email);
  result->email = get_email (email);
  result->flags = get_sex (result->name, &result->last);
  result->name_short[ADDR_NAME_LEN] = '\0';
  strncpy (result->name_short, result->name, ADDR_NAME_LEN);

  if (email != addr && have_special (result->name))
    make_full (result);
  else
    result->full = xstrdup (addr);
  
  result = address_insert (result);

  return result;
}



address_t *
address_construct (const char *email, const char *name, int flags)
{
  address_t *result = xcalloc (1, sizeof (address_t));
  
  result->email = xstrdup (email);
  result->name  = xstrdup (name);
  result->last  = get_last (result->name);
  result->flags = flags;
  result->name_short[ADDR_NAME_LEN] = '\0';
  strncpy (result->name_short, result->name, ADDR_NAME_LEN);
  make_full (result);

  result = address_insert (result);

  return result;
}




void
address_destroy (address_t *addr)
{
  if (addr->name)
    xfree (addr->name);
  if (addr->email)
    xfree (addr->email);
  if (addr->full)
    xfree (addr->full);
  xfree (addr);
}



/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE address.c
 *
 ****************************************************************************/
