#ifndef __ADDRESS_H__
#define __ADDRESS_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/
/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define ADDR_NAME_LEN 16

#define BOOK_ADDRESS 0x100

#define IS_MALE(addr)   (((addr)->flags & 0xff) == 'M')
#define IS_FEMALE(addr) (((addr)->flags & 0xff) == 'F')

/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/

/**
 * This structure describes single email address of form:
 *    "Name Surname <user@host.domain>"
 *    "Nickname <user@host.domain>"
 *    "user@host.domain (Name Surname)"
 */
typedef struct address {
  char *full;           /* whole string */
  char *email;          /* "user@host.domain" */
  char *name;           /* "Name Surname" or "Nickname" */
  char *last;           /* last name - but only pointer, not a copy */
  char  name_short[ADDR_NAME_LEN + 1]; /* same as above but shorter */
  int   flags;          /* first byte is sex - 'M'/'F' */
} address_t;

/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern address_t *address_insert (address_t *addr);
extern address_t *address_create (char *addr);
extern address_t *address_construct (const char *email, const char *name,
                                     int flags);
extern void       address_destroy (address_t *addr);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER address.h
 *
 ****************************************************************************/
#endif
