/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   attachments viewer
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <errno.h>

#include "error.h"
#include "mail.h"
#include "rstring.h"
#include "rmime.h"
#include "select.h"
#include "xmalloc.h"
#include "read.h"
#include "line.h"
#include "cmd.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define WINDOW_WIDTH (COLS - 18)
#define TYPE_WIDTH 20

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static select_t *attach_select = NULL;
static mail_t   *attach_mail   = NULL;
static line_t   *line          = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/

int attach_selected_item = -1;

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static char *
item_string (mime_t *mime)
{
  line_clear (line);
  
  if (mime->type_desc)
    line_add_str_pad (line, mime->type_desc, TYPE_WIDTH);
  else {
    switch (mime->type){

      case MTYPE_UNKNOWN:
        line_add_str_pad (line, "unknown", TYPE_WIDTH);
        break;
    
      case MTYPE_TEXT_PLAIN:
        line_add_str_pad (line, "text/plain", TYPE_WIDTH);
        break;

      case MTYPE_TEXT_OTHER:
        line_add_str_pad (line, "text", TYPE_WIDTH);
        break;

      case MTYPE_HTML:
        line_add_str_pad (line, "text/html", TYPE_WIDTH);
        break;

      case MTYPE_IMAGE:
        line_add_str_pad (line, "image", TYPE_WIDTH);
        break;

      case MTYPE_APPLICATION:
        line_add_str_pad (line, "application", TYPE_WIDTH);
        break;

      case MTYPE_MESSAGE:
        line_add_str_pad (line, "message/rfc2882", TYPE_WIDTH);
        break;
      
      case MTYPE_X_SOMETHING:
        line_add_str_pad (line, "x-type", TYPE_WIDTH);
        break;
    }
  }

  if (mime->file_name){
    line_add_str (line, "  ");
    line_add_str (line, mime->file_name);
  }
  
  return xstrdup (line->str);
}


static void
fill_items (rstring_t *items, rmime_t *rmime)
{
  mime_t **mime;

  for (mime = rmime->array; *mime; mime++){
    rstring_add (items, item_string (*mime));
  }
}



static int
close_window (void)
{
  line_destroy (line);
  rstring_free_strings (attach_select->items);
  rstring_delete (attach_select->items);
  return select_close (attach_select);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
attach_open (mail_t *mail)
{
  rstring_t *items = rstring_create_size (mail->rmime->count + 1);

  attach_mail = mail;
  line        = line_create (WINDOW_WIDTH + 1);
  
  fill_items (items, mail->rmime);
  attach_select = select_open (items, WINDOW_WIDTH, 20, 1, 0,
                               _("attachments"));

  select_show (attach_select);

  if (attach_select == NULL)
    return;

  cmd_state_push (CMD_ATTACH);
}



void
attach_close (void)
{
  close_window ();
  attach_selected_item = -1;
  
  cmd_state_pop ();
}



void
attach_hit (void)
{
  attach_selected_item = close_window ();

  cmd_state_pop ();
}



void
attach_prev (void)
{
  select_prev (attach_select);
}



void
attach_next (void)
{
  select_next (attach_select);
}



void
attach_prev_page (void)
{
  select_prev_page (attach_select);
}



void
attach_next_page (void)
{
  select_next_page (attach_select);
}



void
attach_first (void)
{
  select_first (attach_select);
}



void
attach_last (void)
{
  select_last (attach_select);
}




void
attach_save (void)
{
  mime_t *mime = attach_mail->rmime->array[attach_select->bar_pos];
  char   *file = read_argument (_("file: "), mime->file_name);
  
  if (file == NULL)
    return;
  
  if (mime_save_in_file (mime, attach_mail->place.file_name, file))
    error_ (0, errno, "%s", file);
}



/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE attach.c
 *
 ****************************************************************************/
