/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   mail box selection
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>

#include "select.h"
#include "rstring.h"
#include "xmalloc.h"
#include "gettext.h"
#include "ask.h"
#include "cmd.h"
#include "folder.h"
#include "mybox.h"
#include "error.h"
#include "hook.h"
#include "exec.h"
#include "box_selection.h"
#include "file.h"
#include "wrapbox.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define PREAMBLE do { if (box_select == NULL) return; } while (0)

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

/**
 * This holds an array of directory names.
 */
static rstring_t *true_items = NULL;

static select_t *box_select = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static rstring_t *
items_with_info (rstring_t *items)
{
  int        total;
  int        unread;
  int        i;
  int        len;
  int        maxlen = 0;
  char      *item;
  char      *box;
  rstring_t *result = rstring_create_size (items->count + 1);

  for (i = 0; i < items->count; i++){
    len    = strlen (items->array[i]);
    maxlen = (maxlen > len) ? maxlen : len;
  }
  
  for (i = 0; i < items->count; i++){
    box    = file_with_dir (mybox_dir, items->array[i]);
    total  = wrapbox_mail_count (box, &unread);
    item   = xmalloc (maxlen + 20);
    len    = strlen (items->array[i]);

    memcpy (item, items->array[i], len);
    memset (item + len, ' ', maxlen - len + 3);
    sprintf (item + maxlen + 3, "(%d/%d)", unread, total);
    
    rstring_add (result, item);
    xfree (box);
  }

  return result;
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
box_selection_open (void)
{
  rstring_t *items;
  
  true_items = mybox_subdirs ();
  if (true_items == NULL)
    return;

  items = items_with_info (true_items);

  box_select = select_open (items, 30, 8, 1, 0, _("select box"));

  if (box_select){
    select_show (box_select);
    cmd_state_push (CMD_SELECT_BOX);
  }
}



void
box_selection_next (void)
{
  PREAMBLE;
  
  select_next (box_select);
}



void
box_selection_prev (void)
{
  PREAMBLE;
  
  select_prev (box_select);
}



void
box_selection_next_page (void)
{
  PREAMBLE;
  
  select_next_page (box_select);
}



void
box_selection_prev_page (void)
{
  PREAMBLE;
  
  select_prev_page (box_select);
}



void
box_selection_first (void)
{
  PREAMBLE;
  
  select_first (box_select);
}



void
box_selection_last (void)
{
  PREAMBLE;
  
  select_last (box_select);
}



char *
box_selection_close (void)
{
  int index;

  index = box_select->bar_pos;

  return file_with_dir (mybox_dir, true_items->array[index]);
}



void
box_selection_leave (void)
{
  PREAMBLE;
  
  rstring_free_strings (box_select->items);
  rstring_delete (box_select->items);
  rstring_delete (true_items);
  select_close (box_select);
  box_select = NULL;

  cmd_state_pop ();
}



void
box_selection_hit (void)
{
  char *newbox;

  PREAMBLE;
  
  folder_at_leave ();
  newbox = box_selection_close ();

  if (folder_read_box (newbox) == 0){
    cmd_state_pop ();
    rstring_free_strings (box_select->items);
    rstring_delete (box_select->items);
    rstring_delete (true_items);
    select_close (box_select);
    box_select = NULL;
  }
  else {
    box_selection_re_read ();
  }

  xfree (newbox);
}




void
box_selection_re_read (void)
{
  rstring_t *items;
  
  PREAMBLE;
  
  items = mybox_subdirs ();
  if (items){
    select_install_new_items (box_select, items);
  }
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE box_selection.c
 *
 ****************************************************************************/
