/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------
   
   User-invoked functions are kept in exec_table.t file. This module
   contains routines used to execute them.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "exec.h"
#include "abook.h"
#include "attach.h"
#include "bayes.h"
#include "box_selection.h"
#include "cmd.h"
#include "choose.h"
#include "error.h"
#include "fetch.h"
#include "folder.h"
#include "mailreader.h"
#include "sender.h"
#include "xmalloc.h"
#include "help.h"
#include "gettext.h"
#include "mybox.h"
#include "wrapbox.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static exec_t table[] = {
#include "exec_table.t"
};

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static int
exec_cmp (const void *a, const void *b)
{
  exec_t *aa = (exec_t *) a;
  exec_t *bb = (exec_t *) b;

  return strcmp (aa->name, bb->name);
}



static void
execute (exec_t *exec)
{
  exec->fun ();
  hook_execute (exec->hook);
}



static void
destroy_exec (exec_t *exec)
{
  if (exec->hook){
    hook_destroy (exec->hook);
    xfree (exec->hook);
  }
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
exec_init (void)
{
  qsort (table, sizeof (table) / sizeof (exec_t), sizeof (exec_t), exec_cmp);
}



exec_t *
exec_lookup (char *name)
{
  int lbound = 0;
  int rbound = sizeof (table) / sizeof (exec_t);
  int index;
  int ret;

  if (name == NULL)
    return NULL;
  
  while (rbound > lbound){
    index = (rbound + lbound) / 2;
    ret   = strcmp (name, table[index].name);
    if (ret == 0){
      return table + index;
    }
    if (ret < 0)
      rbound = ((index == rbound) ? rbound - 1 : index);
    if (ret > 0)
      lbound = ((index == lbound) ? lbound + 1 : index);
  }
  return NULL;
}



exec_t *
exec_lookup_fun (void (*fun)(void))
{
  int i;

  for (i = 0; i < sizeof (table) / sizeof (exec_t); i++){
    if (table[i].fun == fun)
      break;
  }
  if (i < sizeof (table) / sizeof (exec_t))
    return table + i;
  return NULL;
}




void
exec_run (exec_t *exec)
{
  if (exec)
    execute (exec);
}



void
exec_run_name (char *name)
{
  exec_t *exec;

  if (name == NULL)
    return;
  
  exec = exec_lookup (name);

  if (exec)
    execute (exec);
  else
    error_ (0, 0, _("undefined function %s"), name);
}



void
exec_free_resources (void)
{
  int i;
  
  for (i = 0; i < sizeof (table) / sizeof (exec_t); i++){
    destroy_exec (table + i);
  }
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE exec.c
 *
 ****************************************************************************/
