/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "keymap.h"
#include "xmalloc.h"
#include "error.h"
#include "exec.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define FIRST_ALLOC 10

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
reallocate (keymap_t *km)
{
  if (km->size == km->count){
    km->size = (km->size + 1) * 2;
    km->keys = xrealloc (km->keys, km->size * sizeof (struct key));
  }
}


static void
execute (struct key *k)
{
  exec_run (k->exec);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
keymap_init (keymap_t *km)
{
  km->size  = FIRST_ALLOC;
  km->count = 0;
  km->keys  = xmalloc (km->size * sizeof (struct key));
}


void
keymap_destroy (keymap_t *km)
{
  if (km->keys)
    xfree (km->keys);
}



struct key *
keymap_lookup (keymap_t *km, int key)
{
  int i;

  for (i = 0; i < km->count; i++){
    if (km->keys[i].key == key)
      return km->keys + i;
  }
  return NULL;
}



void
keymap_add (keymap_t *km, int key, void (*fun)(void))
{
  struct key *k    = keymap_lookup (km, key);
  exec_t     *exec = exec_lookup_fun (fun);

  if (k){
    k->exec = exec;
  }
  else {
    reallocate (km);
    km->keys[km->count].key  = key;
    km->keys[km->count].exec = exec;
    km->count++;
  }
}


void
keymap_action (keymap_t *km, int key)
{
  struct key *k = keymap_lookup (km, key);

  if (k){
    execute (k);
  }
  else {
    error_ (0, 0, "key: %d", key);
  }
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE keymap.c
 *
 ****************************************************************************/
