/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   A Longest Common Subsequence implementation.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "lcs.h"
#include "xmalloc.h"
#include "error.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static int **
int_table (int m, int n)
{
  int   i;
  int **result;
  
  m += 1;
  n += 1;
  result = xmalloc (m * sizeof (int *));
  
  for (i = 0; i < m; i++){
    result[i] = xmalloc (n * sizeof (int));
  }
  return result;
}



static void
destroy_table (int **a, int m, int n)
{
  int i;

  for (i = 0; i < m; i++){
    xfree (a[i]);
  }
  xfree (a);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

lcs_t *
lcs_compute (void *x, void *y, int xsize, int ysize, int xmemb, int ymemb,
             int (*equal)(void *, void *))
{
  lcs_t *lcs = xmalloc (sizeof (lcs_t));
  int    i;
  int    j;

  lcs->m = xmemb;
  lcs->n = ymemb;
  lcs->c = int_table (lcs->m, lcs->n);
  lcs->b = int_table (lcs->m, lcs->n);

  for (i = 1; i <= lcs->m; i++){
    lcs->c[i][0] = 0;
  }
  for (i = 0; i <= lcs->n; i++){
    lcs->c[0][i] = 0;
  }

  for (i = 1; i <= lcs->m; i++){
    for (j = 1; j <= lcs->n; j++){
      if (equal (x + xsize * (i - 1), y + ysize * (j - 1))){
        lcs->c[i][j] = lcs->c[i - 1][j - 1] + 1;
        lcs->b[i][j] = LCS_BOTH;
      }
      else if (lcs->c[i - 1][j] >= lcs->c[i][j - 1]){
        lcs->c[i][j] = lcs->c[i - 1][j];
        lcs->b[i][j] = LCS_UP;
      }
      else {
        lcs->c[i][j] = lcs->c[i][j - 1];
        lcs->b[i][j] = LCS_LEFT;
      }
    }
  }

  return lcs;
}



void
lcs_destroy (lcs_t *lcs)
{
  if (lcs == NULL)
    return;
  destroy_table (lcs->c, lcs->m + 1, lcs->n + 1);
  destroy_table (lcs->b, lcs->m + 1, lcs->n + 1);
  xfree (lcs);
}



int
lcs_length (lcs_t *lcs)
{
  if (lcs == NULL)
    return -1;
  return lcs->c[lcs->m][lcs->n];
}



void
lcs_mark_part (lcs_t *lcs, void *a, void *b, int asize, int bsize,
               void (*fun)(void *, void *))
{
  int i = lcs->m;
  int j = lcs->n;

  while (i && j){
    if (lcs->b[i][j] == LCS_BOTH){
      i--;
      j--;
      fun (a + asize * i, b + bsize * j);
    }
    else if (lcs->b[i][j] == LCS_UP){
      i--;
    }
    else {
      j--;
    }
  }
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE lcs.c
 *
 ****************************************************************************/
