/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   maildir operations
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <string.h>
#include <errno.h>

#include "xmalloc.h"
#include "mail.h"
#include "mlex.h"
#include "file.h"
#include "error.h"
#include "maildir.h"
#include "progress.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static char *directory     = NULL;
static char *directory_cur = NULL;
static char *directory_new = NULL;
//static char *directory_tmp = NULL;


static unsigned last_time = 0;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/

static void free_resources (void);
static int  dir_is_valid (char *dir);
static void free_list (struct dirent **list, int count);
static int  select_fun (const struct dirent *entry);
static void process_list (struct dirent **list, int count, const char *dir);

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/


static unsigned char
char_to_flag (char x)
{
  switch (x){
    case 'R':
      return FLAG_ANSWERED;
      break;

    case 'T':
      return FLAG_TRASHED;
      break;

    case 'D':
      return FLAG_DRAFT;
      break;

    case 'F':
      return FLAG_FLAGGED;
      break;

    case 'S':
      return FLAG_READ;
      break;

    case 'P':
      return FLAG_PASSED;
      break;

    case 'L':
      return FLAG_LEGITIMATE;
      break;
  }
  return 0;
}



static char
flag_to_char (unsigned char flag)
{
  switch (flag){

    case FLAG_READ:
      return 'S';
    
    case FLAG_ANSWERED:
      return 'R';

    case FLAG_TRASHED:
      return 'T';

    case FLAG_DRAFT:
      return 'D';

    case FLAG_FLAGGED:
      return 'F';

    case FLAG_PASSED:
      return 'P';

    case FLAG_LEGITIMATE:
      return 'L';
  }
  return '\0';
}


static void
free_resources (void)
{
  if (directory){
    xfree (directory);
    directory = NULL;
  }
  if (directory_cur){
    xfree (directory_cur);
    directory_cur = NULL;
  }
  if (directory_new){
    xfree (directory_new);
    directory_new = NULL;
  }
}



static int
dir_is_valid (char *dir)
{
  DIR *dp;

  dp = opendir (dir);
  if (!dp)
    return 0;
  closedir (dp);
  return 1;
}



static void
make_dir_names (char *dir)
{
  size_t len    = strlen (dir);

  free_resources ();
  
  directory     = xstrdup (dir);
  directory_cur = xmalloc (len + 5);
  directory_new = xmalloc (len + 5);

  memcpy (directory_cur, dir, len);
  memcpy (directory_new, dir, len);
  memcpy (directory_cur + len, "/cur", 5);
  memcpy (directory_new + len, "/new", 5);
}



static void
free_list (struct dirent **list, int count)
{
  int i;

  for (i = 0; i < count; i++)
    free (list[i]);

  if (count)
    free (list);
}



static int
select_fun (const struct dirent *entry)
{
  return *entry->d_name != '.';
}



static int
select_trashed_fun (const struct dirent *entry)
{
  char *seek;
  
  if (*entry->d_name == '.')
    return 0;
  seek = strrchr (entry->d_name, ',');
  if (seek == NULL)
    return 0;
  return (int) strchr (seek, 'T');
}



static int
select_old_fun (const struct dirent *entry)
{
  char *seek;
  
  if (*entry->d_name == '.')
    return 0;
  seek = strrchr (entry->d_name, ',');
  if (seek == NULL)
    return 1;
  seek = strchr (seek, 'S');
  if (seek == NULL)
    return 1;
  return 0;
}


static time_t
mail_date (const char *fname)
{
  return strtol (fname, NULL, 10);
}




static unsigned char
mail_flags (const char *fname, const char *dir, size_t dlen)
{
  unsigned char flags = 0;
  char *seek;
  
  if (dir[dlen - 1] == 'r')
    flags |= FLAG_OLD;

  seek = strstr (fname, ":");
  while (seek && *seek){
    flags |= char_to_flag (*seek);
    seek++;
  }

  return flags;
}



static void
add_mail (const char *fname, const char *dir)
{
  int     ret;
  time_t  tm;
  size_t  dname_len = strlen (dir);
  char   *full_name;

  tm = mail_date (fname);
/*
  if (!tm)
    return;
*/
  full_name = file_with_dir (dir, fname);
  
  yyin = fopen (full_name, "r");
  if (!yyin){
    xfree (full_name);
    return;
  }

  ret = mlex_scan_file (tm);
  if (ret == NEXT_MAIL){
    newmail->index            = mail_count;
    newmail->flags            = FLAG_MAILDIR;
    newmail->flags           |= mail_flags (fname, dir, dname_len);
    newmail->place.file_name  = full_name;
    mail_insert (newmail);
  }
  else {
    error_ (0, 0, _("%s has bad format"), fname);
    xfree (full_name);
  }

  fclose (yyin);
}



static void
delete_mail (const char *fname, const char *dir)
{
  char *full_name;

  full_name = file_with_dir (dir, fname);
  unlink (full_name);
  xfree (full_name);
}



static void
process_list (struct dirent **list, int count, const char *dir)
{
  int i;

  for (i = 0; i < count; i++){
    add_mail (list[i]->d_name, dir);
    progress_add (1);
  }
}



static void
delete_files (struct dirent **list, int count)
{
  int i;

  for (i = 0; i < count; i++){
    delete_mail (list[i]->d_name, directory_cur);
  }
}



static char *
add_comma (mail_t *mail)
{
  int   len   = strlen (mail->place.file_name);
  char *fname = xmalloc (len + 6);

  memcpy (fname, mail->place.file_name, len);
  memcpy (fname + len, ":2,", 4);

  return fname;
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


int
maildir_is_valid (const char *dir)
{
  size_t len = strlen (dir);
  int    ret;

  char *directory_cur = xmalloc (len + 5);
  char *directory_new = xmalloc (len + 5);

  memcpy (directory_cur, dir, len);
  memcpy (directory_new, dir, len);
  memcpy (directory_cur + len, "/cur", 5);
  memcpy (directory_new + len, "/new", 5);

  ret = dir_is_valid (directory_cur) && dir_is_valid (directory_new);
  xfree (directory_cur);
  xfree (directory_new);
  return ret;
}



int
maildir_read_dir (char *dir)
{
  int             cur_count;
  int             new_count;
  struct dirent **cur_list;
  struct dirent **new_list;
  
  progress_open (_("reading directory"));
  
  make_dir_names (dir);
  if (!dir_is_valid (directory_cur) || !dir_is_valid (directory_new)){
    free_resources ();
    return -1;
  }

  cur_count = scandir (directory_cur, &cur_list, select_fun, alphasort);
  new_count = scandir (directory_new, &new_list, select_fun, alphasort);

  if (cur_count == -1 || new_count == -1){
    free_list (cur_list, cur_count);
    free_list (new_list, new_count);
    free_resources ();
    return -1;
  }

  progress_set_max (cur_count + new_count);
  
  mail_set_array_size (cur_count + new_count + 1);

  process_list (cur_list, cur_count, directory_cur);
  process_list (new_list, new_count, directory_new);

  free_list (cur_list, cur_count);
  free_list (new_list, new_count);
  mail_find_parents ();

  progress_close ();

  return mail_count;
}



void
maildir_free_resources (void)
{
  maildir_remove_trashed ();
  free_resources ();
}



int
maildir_mail_header (mail_t *mail, char **place)
{
  mime_t *mime = mail->rmime->array[0];
  
  if (file_part (mail->place.file_name, 0, mime->offset_start, place)){
    *place = NULL;
    return 1;
  }
  return 0;
}



int
maildir_mail_body (mail_t *mail, char **place, mime_t *mime)
{
  *place = mime_get_and_decode (mime, mail->place.file_name, NULL);
  if (*place == NULL)
    return 1;
  return 0;
}



char *
maildir_valid_file_name (void)
{
  size_t    hlen      = 100;
  unsigned  timestamp = time (NULL);
  char     *hname     = xmalloc (hlen);
  char     *fname;

  while (gethostname (hname, hlen)){
    hlen  = (hlen + 1) * 2;
    hname = xrealloc (hname, hlen);
  }
  hlen = strlen (hname);

  if (timestamp <= last_time){
    timestamp = ++last_time;
  }
  else {
    last_time = timestamp;
  }

  /*              time  .   pid  . hostname \0 */
  fname = xmalloc (10 + 1 + 10 + 1 + hlen + 1);

  sprintf (fname, "%d.%d.%s", timestamp, getpid (), hname);
  xfree (hname);
  return fname;
}


char *
maildir_tmp_file (char *dir)
{
  char   *fname;
  char   *file  = maildir_valid_file_name ();
  size_t  dlen  = strlen (dir);
  size_t  flen  = strlen (file);

  /*               dir    /  tmp  /   file */
  fname = xmalloc (dlen + 1 + 3 + 1 + flen);

  sprintf (fname, "%s/tmp", dir);
  if (!dir_is_valid (fname))
    if (mkdir (fname, 0700)){
      error_ (0, errno, "%s", fname);
      return NULL;
    }

  sprintf (fname, "%s/tmp/%s", dir, file);
  xfree (file);
  return fname;
}



void
maildir_move_to_new (char *fname)
{
  char *newname = xstrdup (fname);
  char *seek    = strstr (newname, "tmp");
  
  if (seek == NULL){
    xfree (newname);
    return;
  }
  
  memcpy (seek, "new", 3);
  if (file_rename (fname, newname))
    error_ (0, errno, "%s", newname);
  xfree (newname);
}



void
maildir_move_to_cur (mail_t *mail)
{
  char   *fname   = mail->place.file_name;
  size_t  len     = strlen (fname);
  char   *newname = xmalloc (len + 4);
  char   *seek;

  memcpy (newname, fname, len + 1);
  seek = strstr (newname, "new");
  if (seek == NULL){
    xfree (newname);
    return;
  }
  memcpy (seek, "cur", 3);

  memcpy (newname + len, ":2,", 4);
  if (file_rename (fname, newname)){
    error_ (0, errno, "%s", newname);
    xfree (newname);
    return;
  }
  xfree (fname);
  mail->place.file_name = newname;
}



void
maildir_add_flag (mail_t *mail, flag_t flag)
{
  char   *fname;
  size_t  len;
  char   *newname;
  char   *seek;
  char    newchar = flag_to_char (flag);

  if (mail->flags & flag)
    return;
  
  mail->flags |= flag;

  if (newchar == '\0')
    return;
  
  maildir_move_to_cur (mail);

  fname = mail->place.file_name;
  len   = strlen (fname);
  seek  = strchr (fname, newchar);
  if (seek)
    return;
  newname = xmalloc (len + 2);
  memcpy (newname, fname, len + 1);
  seek = strrchr (newname, ',');
  if (!seek){
    xfree (newname);
    newname = add_comma (mail);
    len    += 3;
    seek    = strrchr (newname, ',');
  }
  do
    seek++;
  while (*seek && *seek < newchar);
  memmove (seek + 1, seek, newname + len - seek + 1);
  *seek = newchar;

  if (file_rename (fname, newname)){
    error_ (0, errno, "%s", newname);
    xfree (newname);
    return;
  }
  xfree (fname);
  mail->place.file_name = newname;
}



void
maildir_remove_flag (mail_t *mail, flag_t flag)
{
  char   *fname;
  size_t  len;
  char   *newname;
  char   *seek;
  char    newchar = flag_to_char (flag);

  if (!(mail->flags & flag))
    return;
  
  mail->flags &= ~flag;

  if (newchar == '\0')
    return;
  
  maildir_move_to_cur (mail);

  fname   = mail->place.file_name;
  len     = strlen (fname);
  newname = xmalloc (len + 1);
  memcpy (newname, fname, len + 1);
  seek    = strrchr (newname, ',');
  if (!seek){
    xfree (newname);
    newname = add_comma (mail);
    len    += 3;
    seek    = strrchr (newname, ',');
  }
  do
    seek++;
  while (*seek && *seek < newchar);
  memmove (seek, seek + 1, newname + len - seek);

  if (file_rename (fname, newname)){
    error_ (0, errno, "%s", newname);
    xfree (newname);
    return;
  }
  xfree (fname);
  mail->place.file_name = newname;
}



void
maildir_remove_trashed (void)
{
  int             count;
  struct dirent **list;

  if (directory_cur == NULL)
    return;
  
  count = scandir (directory_cur, &list, select_trashed_fun, alphasort);
  delete_files (list, count);
  free_list (list, count);
}



int
maildir_create_dir (char *name)
{
  char *dir;

  mkdir (name, 0700);
  
  dir = file_with_dir (name, "cur");
  if (mkdir (dir, 0700)){
    error_ (0, errno, "%s", dir);
    xfree (dir);
    return 1;
  }
  xfree (dir);

  dir = file_with_dir (name, "new");
  if (mkdir (dir, 0700)){
    error_ (0, errno, "%s", dir);
    xfree (dir);
    return 1;
  }
  xfree (dir);

  dir = file_with_dir (name, "tmp");
  if (mkdir (dir, 0700)){
    error_ (0, errno, "%s", dir);
    xfree (dir);
    return 1;
  }
  xfree (dir);
  return 0;
}



int
maildir_move_mail_to (mail_t *mail, char *dir)
{
  char *dest_dir;
  char *dest_file;
  char *destination;
  
  if (!(mail->flags & FLAG_MAILDIR))
    return 1;

  if (mail->flags & FLAG_READ || mail->flags & FLAG_OLD)
    dest_dir = file_with_dir (dir, "cur");
  else
    dest_dir = file_with_dir (dir, "new");

  dest_file   = maildir_valid_file_name ();
  destination = file_with_dir (dest_dir, dest_file);

  if (file_rename (mail->place.file_name, destination)){
    error_ (0, errno, "%s", dest_file);
    xfree (dest_dir);
    xfree (dest_file);
    xfree (destination);
    return 1;
  }

  xfree (mail->place.file_name);
  xfree (dest_dir);
  xfree (dest_file);
  mail->place.file_name = destination;

  return 0;
}



int
maildir_mail_size (mail_t *mail)
{
  struct stat st;

  if (stat (mail->place.file_name, &st)){
    error_ (0, errno, "%s", mail->place.file_name);
    return 0;
  }

  return st.st_size;
}



int
maildir_mail_count (const char *box, int *unread)
{
  char           *cur = file_with_dir (box, "cur");
  char           *new = file_with_dir (box, "new");
  struct dirent **cur_list;
  struct dirent **new_list;
  int             cur_count;
  int             new_count;
  int             total_count;

  cur_count = scandir (cur, &cur_list, select_fun, alphasort);
  new_count = scandir (new, &new_list, select_fun, alphasort);

  if (cur_count == -1 || new_count == -1){
    free_list (cur_list, cur_count);
    free_list (new_list, new_count);
    if (unread)
      *unread = 0;
    return 0;
  }

  free_list (cur_list, cur_count);
  free_list (new_list, new_count);

  total_count = new_count + cur_count;

  cur_count = scandir (cur, &cur_list, select_old_fun, alphasort);
  free_list (cur_list, cur_count);
  
  
  if (unread)
    *unread = new_count + cur_count;

  xfree (cur);
  xfree (new);
  
  return total_count;
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE maildir.c
 *
 ****************************************************************************/
