/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------


   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "ecurses.h"
#include "mail.h"
#include "mailinfo.h"
#include "line.h"
#include "screen.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static WINDOW *win  = NULL;
static mail_t *mail = NULL;

static line_t *line = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
display (void)
{
  int i;

  line_put_str (line, "    From: ");
  if (mail->from && mail->from->full)
    line_add_str_dots (line, mail->from->full);
  mvwaddstr (win, 0, 0, line->str);
  
  line_put_str (line, " Subject: ");
  if (mail->subject)
    line_add_str_dots (line, mail->subject);
  mvwaddstr (win, 1, 0, line->str);

  line_put_str (line, "      To: ");
  if (mail->to && mail->to->count){
    line_add_str_dots (line, mail->to->array[0]->full);
    for (i = 1; i < mail->to->count; i++){
      line_add_str_dots (line, ", ");
      line_add_str_dots (line, mail->to->array[i]->full);
    }
  }
  mvwaddstr (win, 2, 0, line->str);
  
  line_put_str (line, "Reply-To: ");
  if (mail->reply_to && mail->reply_to != mail->from && mail->reply_to->full)
    line_add_str_dots (line, mail->reply_to->full);
  mvwaddstr (win, 3, 0, line->str);

  line_clear (line);
  mvwaddstr (win, 4, 0, line->str);
  mvwaddstr (win, 5, 0, line->str);
  mvwaddstr (win, 6, 0, line->str);

  line_add_str (line, _("            Attachments"));
  wbkgdset (win, COLOR_PAIR (1));
  mvwaddstr (win, 7, 0, line->str);
  wbkgdset (win, 0);
  
  screen_wnoutrefresh (win);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
mailinfo_show (mail_t *m)
{
  win  = newwin (MAILINFO_LINES, COLS, 0, 0);
  mail = m;
  line = line_create (COLS + 1);
  
  display ();
}


void
mailinfo_refresh (void)
{
  display ();
}


void
mailinfo_close (void)
{
  delwin (win);
  line_destroy (line);
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mailinfo.c
 *
 ****************************************************************************/
