/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   This is very simple module used to get full list of messages from
   file in mbox format.

*/

#define _GNU_SOURCE 1


/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>

#include "xmalloc.h"
#include "mail.h"
#include "mbox.h"
#include "hash.h"
#include "mlex.h"
#include "file.h"
#include "error.h"
#include "progress.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static char *file_name = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/

static void allocate_resources (char *file);
static void free_resources (void);

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/


static void
allocate_resources (char *file)
{
  file_name = xstrdup (file);
}


static void
free_resources (void)
{
  if (file_name){
    xfree (file_name);
    file_name = NULL;
  }
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


int
mbox_may_be_valid (char *fname)
{
  struct stat st;

  if (stat (fname, &st)){
    error_ (0, errno, fname);
    return 0;
  }

  if (S_ISREG (st.st_mode))
    return 1;
  return 0;
}



int
mbox_mail_size (mail_t *mail)
{
  mime_t *lastmime;

  lastmime = mail->rmime->array[mail->rmime->count - 1];
  return lastmime->offset_end - mail->place.offset_header;
}
    


int
mbox_read_file (char *file)
{
  struct stat st;
  int         ret = 0;

  if (stat (file, &st)){
    error_ (0, errno, file);
    return -1;
  }

  if (!S_ISREG (st.st_mode))
    return -1;
  
  progress_open (_("reading file"));
  progress_set_max (st.st_size);
  
  yyin = fopen (file, "r");
  if (!yyin){
    error_ (0, errno, file);
    progress_close ();
    return -1;
  }
  
  free_resources ();
  allocate_resources (file);

  /* first invocation of mbox_scan tells us only if we are reading
     a mailbox; it does not bring any information yet */
  if (mlex_mbox_scan_start () != NEXT_MAIL){
    fclose (yyin);
    free_resources ();
    progress_close ();
    return -1;
  }

  while (ret != END_OF_FILE){
    
    ret = mlex_scan ();

    switch (ret){

      case NEXT_MAIL:
        newmail->index = mail_count;
        mail_insert (newmail);
        progress_add (mbox_mail_size (newmail));
        break;

      case BROKEN_MAIL:
        fclose (yyin);
        free_resources ();
        progress_close ();
        return -1;
        break;

    }
  }
  progress_close ();
  fclose (yyin);
  mail_find_parents ();
  return mail_count;
}



int
mbox_mail_header (mail_t *mail, char **place)
{
  mime_t *mime = mail->rmime->array[0];
  
  if (file_part (file_name, 0, mime->offset_start, place)){
    *place = NULL;
    return 1;
  }
  return 0;
}



int
mbox_mail_body (mail_t *mail, char **place, mime_t *mime)
{
  *place = mime_get_and_decode (mime, file_name, NULL);
  if (*place == NULL)
    return 1;
  return 0;
}



void
mbox_free_resources (void)
{
  free_resources ();
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mbox.c
 *
 ****************************************************************************/
