#ifndef __MBOX_H__
#define __MBOX_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------
   
   1. What do we need to do?

   We'd like to have some kind of list containig all e-mails from file
   in mbox format.  Every item on the list must contain basic info. It
   should use small amount of memory and must be easy and fast to build.
   This structure should provide fast way of creating tree, where one
   mail is child of another if it is its reply.


   2. How are we going to make it?

   We're going to make an array containig mails, and a binary tree
   containing their identifiers.  So I would say (for now), that
   we need about O(nlogn) time to achieve our goal, which is IMHO
   quite fast.  The tree itself does not have any pointer to array.
   This is because the array may be reallocated into another block
   of memory which would cause destruction of tree.


   3. Why not hash table?

   Of course I could use hash table instead (requiring only O(n) time),
   but tree is easier to maintain, and does not work much longer.
   Problem with hash table is that one have to rehash it once it is too
   small to work as fast as it is expected to do, and if one wants to
   have growing hash table in his program, he has to implement it first.


   4. How to use it?

   Call mbox_read_file to fill mail_array.  You can then use
   mbox_mail_header and mbox_mail_body to get string containing header or
   body of mail.  These two functions return pointer to statically
   allocated block of memory, so after next call to any of them, old
   information is lost.  You can call mbox_free_resources after you are
   not going to use this module any more.  You do not have to call
   mbox_free_resources before other mbox_* function calls.  It will not
   result in any resource leak.

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include "mail.h"
#include "mime.h"

/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern int  mbox_may_be_valid (char *fname);
extern int  mbox_read_file (char *name);
extern int  mbox_mail_header (mail_t *mail, char **place);
extern int  mbox_mail_body (mail_t *mail, char **place, mime_t *mime);
extern void mbox_free_resources (void);
extern int  mbox_mail_size (mail_t *mail);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER mbox.h
 *
 ****************************************************************************/
#endif
