#ifndef __NETWORKING_H__
#define __NETWORKING_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 bazyl, rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   1. Opening connection.

   There are 2 ways of establishing tcp connection with server.  There are
   only few situations in which you could want to use net_open_connection
   instead of net_open_service.  First is when you run your local service
   and you cannot list it in your /etc/services.  Second is when you
   explicity want to use non-standard port number.  Otherwise you are
   strongly encouraged to use net_open_service.
   These two functions create a socket that is ready to use.  Connection
   to a server is established and you are ready to send and receive data.

   
   2. Sending / receiving.

   net_send and net_recv are only simple wrappers for original send and recv
   functions (see manual).  net_send_txt needs that its argument is
   null-terminated.  It sends buffer without trailing \0 byte.


   3. More complex: net_exchange_msg

   Most of protocols are designed to be similar to human talk.  SMTP and
   POP3 are examples of such protocols.  This functions tries to simplify
   these operations.  If FMT argument is not null, then it will be a format
   for text to send.  Next arguments will be inserted into it in printf
   style.  Then function will try to receive a response from server.  It will
   use function is_complete (giving it buffer and its size) to determine
   wheter reply is complete or it should continue receiving.  After all it
   will give parse_fun a buffer as an input and return its return value.

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include <unistd.h>
#include <netinet/in.h>

/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/

extern struct in_addr net_local_address;
extern unsigned short net_local_port;

/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern void net_free_resources (void);
extern int  net_open_connection (const char *hostname, unsigned short port);
extern void net_close_connection (void);
extern int  net_send (const void *msg, size_t len);
extern int  net_recv (void *buf, size_t len);
extern int  net_send_txt (const char *buf);
extern int  net_exchange_msg (int (*parse_fun)(char *),
                              int (*is_complete)(char *, size_t),
                              char *fmt, ...)
#ifdef __GNUC__
  __attribute__ ((format (printf, 3, 4)))
#endif
  ;


/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER networking.h
 *
 ****************************************************************************/
#endif
