/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "address.h"
#include "rarray.h"
#include "raddress.h"
#include "rstring.h"
#include "xmalloc.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

raddress_t *
raddress_create_size (int size)
{
  return (raddress_t *) rarray_create_size (size);
}




raddress_t *
raddress_create (void)
{
  return (raddress_t *) rarray_create_size (2);
}




void
raddress_destroy (raddress_t *ptr)
{
  rarray_destroy ((rarray_t *) ptr);
}



void
raddress_add (raddress_t *ptr, address_t *x)
{
  rarray_add ((rarray_t *) ptr, x);
}



void
raddress_remove (raddress_t *ptr, unsigned index)
{
  rarray_remove ((rarray_t *) ptr, index);
}



void
raddress_shrink (raddress_t *ptr)
{
  rarray_shrink ((rarray_t *) ptr);
}



raddress_t *
raddress_join (raddress_t *x, raddress_t *y)
{
  return (raddress_t *) rarray_join ((rarray_t *) x, (rarray_t *) y);
}



void
raddress_add_array (raddress_t *x, raddress_t *y)
{
  rarray_add_array ((rarray_t *) x, (rarray_t *) y);
}


raddress_t *
raddress_get_from_header (char *header)
{
  raddress_t *result;
  address_t  *addr;
  char       *seek  = header;
  char       *comma = NULL;
  
  result = raddress_create ();

  while (seek && *seek){
    comma = strchr (seek, ',');
    if (comma)
      *comma = '\0';
  
    addr = address_create (seek);
    raddress_add (result, addr);

    if (comma){
      *comma = ',';
      seek   = comma + 1;
    }
    else
      break;
  }
  
  return result;
}



rstring_t *
raddress_to_strings (raddress_t *ptr)
{
  rstring_t  *result = rstring_create_size (ptr->count + 1);
  address_t **addr;

  for (addr = ptr->array; *addr; addr++){
    rstring_add (result, (*addr)->full);
  }

  return result;
}



/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE raddress.c
 *
 ****************************************************************************/
