/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <string.h>

#ifdef HAVE_GLOB_H
# include <glob.h>
#endif

#include "ecurses.h"
#include "screen.h"
#include "clock.h"
#include "xmalloc.h"
#include "cmd.h"
#include "choose.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static int     prompt_len = 0;
static int     cancel     = 0;
static int     tab        = 0;
static glob_t  file_glob;

static char *buffer   = NULL;
static char *position = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
kill_line (void)
{
  *position = '\0';
  wclrtoeol (cmd_echo_area);
}


static void
next_char (void)
{
  int y, x;
  
  if (*position == '\0')
    return;
  position++;
  getyx (cmd_echo_area, y, x);
  wmove (cmd_echo_area, y, x + 1);
}


static void
prev_char (void)
{
  int y, x;
  
  if (position == buffer)
    return;
  position--;
  getyx (cmd_echo_area, y, x);
  wmove (cmd_echo_area, y, x - 1);
}


static void
first_char (void)
{
  position = buffer;
  wmove (cmd_echo_area, 0, prompt_len);
}



static void
last_char (void)
{
  int y, x;

  getyx (cmd_echo_area, y, x);
  while (*position != '\0'){
    x++;
    position++;
  }
  wmove (cmd_echo_area, y, x);
}



static void
backspace (void)
{
  int y, x;
  int len;

  if (position == buffer)
    return;
  
  len = strlen (position);
  position--;
  memmove (position, position + 1, len + 1);
  getyx (cmd_echo_area, y, x);
  wmove (cmd_echo_area, y, x - 1);
  wclrtoeol (cmd_echo_area);
  mvwaddstr (cmd_echo_area, y, x - 1, position);
  wmove (cmd_echo_area, y, x - 1);
}



static void
kill_char (void)
{
  if (*position == '\0')
    return;
  next_char ();
  backspace ();
}



static void
echo_char (int c)
{
  int y, x;
  int len;
  
  if (*position == '\0'){
    waddch (cmd_echo_area, c);
    *position = c;
    position++;
    *position = '\0';
  }
  else {
    len = strlen (position);
    memmove (position + 1, position, len + 1);
    *position = c;
    position++;

    getyx (cmd_echo_area, y, x);
    wclrtoeol (cmd_echo_area);
    waddstr (cmd_echo_area, position - 1);
    wmove (cmd_echo_area, y, x + 1);
  }
}



/**
 * This is highly unefficient.  MUST be changed someday.
 */
static void
echo_string (char *str, int len)
{
  while (*str && len){
    echo_char (*str);
    str++;
    len--;
  }
}



#ifdef HAVE_GLOB_H
static int
longest_common_prefix (glob_t *vect, int len)
{
  int i;
  int index;

  if (vect->gl_pathc == 1){
    return strlen (vect->gl_pathv[0]);
  }
  
  index = len;
  while (vect->gl_pathv[0][index]){
    for (i = 1; i < vect->gl_pathc; i++){
      if (vect->gl_pathv[0][index] != vect->gl_pathv[i][index])
        break;
    }
    if (i < vect->gl_pathc)
      break;
    index++;
  }
  return index - len;
}
#endif


static void
complete_file_name (void)
{
#ifdef HAVE_GLOB_H
  glob_t  vector;
  int     ret;
  int     len     = position - buffer;
  char   *pattern = xmalloc (len + 2);

  memcpy (pattern, buffer, len);
  pattern[len]     = '*';
  pattern[len + 1] = '\0';

  ret = glob (pattern, 0, NULL, &vector);

  if (ret && ret != GLOB_NOMATCH){
    globfree (&vector);
    xfree (pattern);
    return;
  }

  if (ret == GLOB_NOMATCH){
    fprintf (stderr, "%c", '\a');
    globfree (&vector);
    xfree (pattern);
    return;
  }

  ret = longest_common_prefix (&vector, len);
  if (ret == 0){
    tab = 1;
    fprintf (stderr, "%c", '\a');
    file_glob = vector;
  }
  else if (len < strlen (vector.gl_pathv[0])) {
    echo_string (vector.gl_pathv[0] + len, ret);
    globfree (&vector);
  }

  xfree (pattern);
#endif
}



static void
choose_wrapper (void (*fun)(void))
{
  int y, x;

  getyx (cmd_echo_area, y, x);
  fun ();
  wmove (cmd_echo_area, y, x);
}



static void
leave_choose (void)
{
  if (tab){
    tab = 0;
    globfree (&file_glob);
    choose_wrapper (choose_close);
    cmd_refresh ();
  }
}



static void
action (int c)
{
  switch (c){

    case 1:   /* ^A */
    case KEY_HOME:
      if (tab){
        choose_wrapper (choose_first);
      }
      else {
        first_char ();
      }
      break;
    
    case 2:   /* ^B */
    case KEY_LEFT:
      leave_choose ();
      prev_char ();
      break;

    case 4:   /* ^D  */
    case 330: /* del */
      leave_choose ();
      kill_char ();
      break;
    
    case 5:   /* ^E */
    case KEY_END:
      if (tab){
        choose_wrapper (choose_last);
      }
      else {
        last_char ();
      }
      break;
    
    case 6:   /* ^F */
    case KEY_RIGHT:
      leave_choose ();
      next_char ();
      break;

    case 7:   /* ^G  */
    case 27:  /* esc */
      if (tab){
        leave_choose ();
      }
      else {
        cancel = 1;
      }
      break;

    case 9:   /* TAB */
      if (tab){
        int y, x;
        getyx (cmd_echo_area, y, x);
        choose_open (file_glob.gl_pathc, file_glob.gl_pathv);
        wmove (cmd_echo_area, y, x);
      }
      else {
        complete_file_name ();
      }
      break;
    
    case 11:  /* ^K */
      leave_choose ();
      kill_line ();
      break;

    case 14:  /* ^N */
    case KEY_DOWN:
      if (tab){
        choose_wrapper (choose_next);
      }
      break;

    case 16:  /* ^P */
    case KEY_UP:
      if (tab){
        choose_wrapper (choose_prev);
      }
      break;


    case 263: /* backspace */
      leave_choose ();
      backspace ();
      break;

    case '\r':
      echo_string (file_glob.gl_pathv[choose_index ()],
                   strlen (file_glob.gl_pathv[choose_index ()]));
      leave_choose ();
      break;

    default:
      leave_choose ();
      echo_char (c & 0xff);
      break;
  }
}


static void
write_prompt (const char *prompt)
{
  int y;
  
  mvwaddstr (cmd_echo_area, 0, 0, prompt);
  getyx (cmd_echo_area, y, prompt_len);
  waddstr (cmd_echo_area, buffer);
}



static void
get_string (const char *prompt)
{
  int c;
  
  tab = 0;
  write_prompt (prompt);
  while (1){
    c = wgetch (cmd_echo_area);
    if (c == '\r' && tab == 0){
      break;
    }

    action (c);
    screen_doupdate ();
    if (cancel){
      break;
    }
  }
}



static void
init_buffer (const char *def)
{
  int len;
  
  wbkgd (cmd_echo_area, COLOR_PAIR (8));

  if (buffer == NULL)
    buffer = xmalloc (COLS + 2);

  if (def){
    len = strlen (def);
    memcpy (buffer, def, len + 1);
    position = buffer + len;
  }
  else {
    position  = buffer;
    *position = '\0';
  }
}



static void
clear_win (void)
{
  wmove (cmd_echo_area, 0, 0);
  wclrtoeol (cmd_echo_area);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

char *
read_argument (const char *prompt, const char *def)
{
  init_buffer (def);
  clear_win ();
  
  clock_stop ();
  curs_set (1);
  meta (cmd_echo_area, TRUE);
  get_string (prompt);
  clear_win ();
  curs_set (0);
  clock_resume ();

  if (cancel){
    cancel = 0;
    return NULL;
  }
  return buffer;
}



void
read_free_resources (void)
{
  if (buffer){
    xfree (buffer);
    buffer = NULL;
  }
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE read.c
 *
 ****************************************************************************/
