#ifndef __XMALLOC_H__
#define __XMALLOC_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 bazyl, rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   1. What is this?

   These are wrappers for malloc-related functions.  They always check if
   corespondent function did not return NULL in which case they just finish
   program with error message.  They can also find memory leaks in your
   program.


   2. How can I find my memory leaks?

   It is very simple.  Compile this module with -DXMALLOC_DEBUG option, then
   define environment variable XMALLOC_TRACE, so that it is a valid name of
   file which can be overwritten.  Run your program, and after it finishes
   use mtrace program (which is part of GNU libc).

        $ export XMALLOC_TRACE=xmalloc.log
        $ ./exampl
        $ mtrace exampl xmalloc.log
        No memory leaks.

   Try using it.  You'll like it.

   Oh!  Did I mention, that GNU libc is essential for this to work?

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include <stdlib.h>

/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern void *xmalloc (size_t n);
extern void *xcalloc (size_t n, size_t s);
extern void *xrealloc (void *p, size_t n);
extern char *xstrdup (const char *str);
extern void  xfree (void *ptr);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER xmalloc.h
 *
 ****************************************************************************/
#endif
